# Invoking and Configuring

TL;DR: For the impatient, jump to section [Examples](#markdown-header-examples) section below. 

## Invoking VDB from Command Line

```
stx --load jv:vdb --run VDBStartup
```
You may append VDB command line options and parameters after `--run VDBStartup`,
for example: `stx --load jv:vdb --run VDBStartup --help`.

See below for documentation on command line options and examples

## Invoking VDB from Smalltalk/X IDE

The easiest (and recommended) way to invoke VDB from Smalltalk/X IDE is to use VDB's command line 
launcher:

```
Smalltalk loadPackage: 'jv:vdb'.
VDBStartup main: {}
```
The parameter to `#main:` is an array of command line arguments, for example:
`VDBStartup main: { '--help' }`

See below for documentation on command line options and examples

## Command Line Options

```
usage: vdb [OPTIONS] [PROGRAM [ARGS]] 
       vdb [OPTIONS] [PID]

options:
 --replay ..................... replay last rr record
 --no-pty ..................... do not use PTY-based GDB console even if current
                                platform supports it. 
 -d | --gdb CMD ............... use CMD to launch GDB (instead of platform
                                default).    
 --preference FILE ............ read user settings from FILE
 --no-preferences ............. do not read user settings at all
 --help ....................... output this message
```

## Configuration 

### GDB Configuration

Since VDB de-facto requires a special GDB (until all required patches are 
upstreamed which may take time), most users if not all need to configure 
command to launch GDB. To do so, set `#gdbCommand:` user preference by
evaluating following in workspace:  

```
"/ Set gdb command:
UserPreferences current gdbCommand: `~/path/to/gdb'. 

"/ Save user preferences to make the setting persistent across
"/ sessions:
UserPreferences current save.
```

If no gdb command is configured, VDB uses the default GDB found along the
path (and - on Windows - at some standard places). 

The `#gdbCommand:` settings value can be an arbitrary shell command (or - on
Windows - `cmd.exe` command). This means you may pass additional command line
arguments or even run GDB through remote shell (see the *Examples* section).
This is particularly handy when a  manually compiled GDB is used right from
the build tree (i.e, without installing it using `make install` or alike). In
that case you may want to pass path to  GDB data directory:

```
UserPreferences current gdbCommand: `/home/user/src/binutils-gdb/gdb/gdb --data-directory /home/user/src/binutils-gdb/gdb/data-directory'.
```

### VDB Configuration

Right after VDB initializes and launches GDB, it executes a startup script 
`$HOME/.vdbinit.st` (`%UserProfile%\.vdbinit.st` on Windows). 

Startup script is uses standard Smalltalk syntax and may contain any Smalltalk
code and use any class and/or method or even load new classes. Reference to VDB
(as an instance of `VDBDebuggerApplication` class) is accessible via pre-define 
variable `vdb`  Reference to running GDB (as an instance of `GDBDebugger` is 
accessible through VDB via `vdb debugger`. 

An example of `.vdbinit.st` script:

```
"/ VDB init script
"/ Current VDB instance can be accessed in via variable `vdb`
"/
"/ Set prompt
vdb debugger setParameter: 'prompt' to: '(vdb) '

"/ Set default font:
VDBAbstractApplication defaultTextFont: (CodeView defaultFont asSize: 16).
```

## Examples

1.  Run VDB on local (Linux) machine connecting to remote GDB running on 
    *HiFive Unleashed* board debugging `compilertest`:

    ```
    VDBStartup main: { '--no-pty' . '--gdb' . 'ssh debian-sid-rv64 /home/jv/Projects/gdb/users_jv_vdb_riscv64-linux-gnu/gdb/gdb --data-directory /home/jv/Projects/gdb/users_jv_vdb_riscv64-linux-gnu/gdb/data-directory /home/jv/Projects/J9/omr-riscv/build/fvtest/compilertest' }
    ```

    Here we use:
     
      * `--gdb` option to configure custom command to launch GDB on remote machine 
        using SSH. Note that this requires SSH key-based authentication since VDB
        is completely unaware of any SSH in between and cannot ask interactively 
        for password. 
      * `--no-pty` *MUST* be used because GDB itself runs on remote machine and
        therefore cannot access or allocate PTYs on that remote machine. Again,
        we have to tell VDB this via `--no-pty` option because VDB is unaware of SSH
        being used. 
